/*Ŀ
   void initialize_mode(int video_mode)                                   
  Ĵ
   Initializes a predefined BIOS video mode using the BIOS interrupt      
   service 10H.                                                           
                                                                          
   Accepts an integer value specifing a BIOS video mode to                
   Initialize.                                                            
                                                                          
   Returns nothing.                                                       
  */

    void initialize_mode(int video_mode)
    {
        union REGS r;

        r.h.ah=0x00;
        r.h.al=video_mode;
        int86(0x10,&r,&r);
    }

/*Ŀ
   void cls(void)                                                         
  Ĵ
   Clears the entire contents of both text and graphics video buffer      
   memory locations, effectively blanking the screen for either mode.     
                                                                          
   Accepts nothing.                                                       
                                                                          
   Returns nothing.                                                       
  */

    void cls(void)
    {
        char far *video_pointer;

        video_pointer=MK_FP(0xB800,0x0000);
        _fmemset(video_pointer,0,4000);
        video_pointer=MK_FP(0xA000,0x0000);
        _fmemset(video_pointer,0,(size_t)64000L);
    }

/*Ŀ
   void set_dac(int dac,char r,char g,char b)                             
  Ĵ
   Sets new red, green and blue intensity values for the specified DAC    
   register.                                                              
                                                                          
   Accepts an integer value in range of 0 to 255 specifying the DAC color 
   to modify.                                                             
   Accepts a red value in range of 0 to 63.                               
   Accepts a green value in range of 0 to 63.                             
   Accepts a blue value in range of 0 to 63.                              
                                                                          
   Returns nothing.                                                       
  */

    void set_dac(int dac,char r,char g,char b)
    {
        outportb(0x3C8,dac);
        outportb(0x3C9,r);
        outportb(0x3C9,g);
        outportb(0x3C9,b);
    }

/*Ŀ
   void set_palette(char palette[768])                                    
  Ĵ
   Sets the entire 256 color MCGA palette with the RGB triplets stored    
   in the palette string.                                                 
                                                                          
   Accepts a character string of 768 bytes in length which specifies RGB  
   triplets for each of the 256 colors.                                   
                                                                          
   Returns nothing.                                                       
  */

    void set_palette(char palette[768])
    {
        int n;

        wait_retrace();
        for(n=0;n<768;n+=3)
        {
            set_dac((n/3),palette[n],palette[n+1],palette[n+2]);
        }
    }

/*Ŀ
   void fade_up(char palette[768])                                        
  Ĵ
   Fades the currently active 256 color palette up to the new RGB values  
   specified in the palette string.                                       
                                                                          
   Accepts a character string of 768 bytes in length which specifies RGB  
   triplets for each of the 256 colors to be faded to.                    
                                                                          
   Returns nothing.                                                       
  */

    void fade_up(char palette[768])
    {
        int n,done=0,i=0;
        char temp_palette[768];

        outportb(0x3C7,0);
        for(n=0;n<768;n++)
        {
            temp_palette[n]=inportb(0x3C9);
        }
        while(!done)
        {
            for(n=0;n<768;n++)
            {
                if(temp_palette[n]<palette[n])
                {
                    temp_palette[n]++;
                }
            }
            set_palette(temp_palette);
            i++;
            if(i>=63) done=1;
        }
    }

/*Ŀ
   void fade_down(char palette[768])                                      
  Ĵ
   Fades the currently active 256 color palette down to the new RGB values
   specified in the palette string.                                       
                                                                          
   Accepts a character string of 768 bytes in length which specifies RGB  
   triplets for each of the 256 colors to be faded to.                    
                                                                          
   Returns nothing.                                                       
  */

    void fade_down(char palette[768])
    {
        int n,done=0,i=0;
        char temp_palette[768];

        outportb(0x3C7,0);
        for(n=0;n<768;n++)
        {
            temp_palette[n]=inportb(0x3C9);
        }
        while(!done)
        {
            for(n=0;n<768;n++)
            {
                if(temp_palette[n]>palette[n])
                {
                    temp_palette[n]--;
                }
            }
            set_palette(temp_palette);
            i++;
            if(i>=63) done=1;
        }
    }

/*Ŀ
   void put_pixel(int x,int y,unsigned char color)                        
  Ĵ
   Sets a pixel at the designated X and Y coordinates on the screen.      
                                                                          
   Accepts an X value in range of 0 to 319.                               
   Accepts a Y value in range of 0 to 200.                                
   Accepts a color value in range of 0 to 255.                            
                                                                          
   Returns nothing.                                                       
  */

    void put_pixel(int x,int y,unsigned char color)
    {
        unsigned int offs;

        offs=(x+(y*320));
        asm mov ax,0xA000;
        asm mov es,ax;
        asm mov di,[offs];
        asm mov al,[color];
        asm mov es:[di],al;
    }

/*Ŀ
   void wait_retrace(void)                                                
  Ĵ
   Waits for the monitors vertical retrace period to begin.               
                                                                          
   Accepts nothing.                                                       
                                                                          
   Returns nothing.                                                       
  */

    void wait_retrace(void)
    {
        asm mov dx,0x3DA;
        wait1:
        asm in al,dx;
        asm test al,8;
        asm jne wait1;
        wait2:
        asm in al,dx;
        asm test al,8;
        asm je wait2;
    }
